// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibuild;

/// Split [str] into lines and prefixes them with [prefix].
/// Returns all lines joined together using line endings.
String _prefixEachLine(String str, String prefix, bool lastLineFeed) {
  if (str.isEmpty) {
    return '';
  }

  // Prefex line-by-line.
  var out = new StringBuffer();
  var lines = LineSplitter.split(str);
  lines.forEach((String line) {
    out.writeln('$prefix$line');
  });

  // Remove the terminating line feed to make life easier.
  var prefixed = out.toString();
  return lastLineFeed ? prefixed.substring(0, prefixed.length - 1) : prefixed;
}

/// Recursive method for test if a given string represents a JSON compatible
/// type in Dart.
///
/// Allowed types:
/// - num
/// - int
/// - double
/// - bool
/// - String
/// - List<$allowedType>
/// - Map<String, $allowedType>
bool _isJsonType(String type) {
  // Note that an enumeration is passed as integer.
  if (['num', 'int', 'double', 'bool', 'String'].contains(type)) {
    return true;
  } else {
    if (type.startsWith('List<')) {
      return _isJsonType(type.substring(5, type.length - 1));
    } else if (type.startsWith('Map<String,')) {
      return _isJsonType(type.substring(11, type.length - 1).trimLeft());
    } else {
      return false;
    }
  }
}

/// Create reverse constructor for the given [type] where [data] is used to
/// access the object that contains the data to construct this type and
/// [jsonType] represents the JSON representation of [type].
///
/// [depth] represents the recursive level. This is required to name the List
/// generator iteration index variable.
///
/// Note that impure Maps and nested enumerations are not supported (both are
/// hard to reversely construct and should therefore be avoided).
String _createReverseConstructor(
    String type, String data, String jsonType, int depth) {
  if (_isJsonType(type)) {
    // JSON data can be casted directly.
    return '$data as $type';
  } else if (type.startsWith('List<')) {
    // Impure Lists can be constructed using this custom code.

    // Unwrap the List<...> type.
    var innerType = type.substring(5, type.length - 1);

    // Unwrap the JSON type. This asserts the JSON type uses List as well which
    // should indeed be the case. If this is not the case the produced code will
    // likely contain errors and it's likely it cannot be formatted either.
    var innerJsonType = jsonType.substring(5, jsonType.length - 1);

    // Generate constructor for List memebers.
    var innerConstructor = _createReverseConstructor(
        innerType, '$data[i$depth]', innerJsonType, depth + 1);

    return "new $type.generate($data.length, (int i$depth) => $innerConstructor)";
  } else {
    // Fallback to .fromJson constructor.
    return 'new $type.fromJson($data as $jsonType)';
  }
}
